#include "framebuffer.h"
#include "v3.h"
#include "math.h"

FrameBuffer::FrameBuffer(int u0, int v0, 
  int _w, int _h) : Fl_Gl_Window(u0, v0, _w, _h, 0) {

  w = _w;
  h = _h;
  pix = new unsigned int[w*h];
  Set(0xFFFFFFFF);

}


void FrameBuffer::draw() {

  glDrawPixels(w, h, GL_RGBA, 
    GL_UNSIGNED_BYTE, pix);

}

void FrameBuffer::Set(unsigned int color) {

  for (int i = 0; i < w*h; i++)
    pix[i] = color;

}

void FrameBuffer::Set(int u, int v, unsigned int col) {

  pix[(h-1-v)*w+u] = col;

}

void FrameBuffer::SetGuarded(int u, int v, unsigned int col) {

  if (u < 0 || u > w-1 || v < 0 || v > h-1)
    return;
  Set(u, v, col);

}

void FrameBuffer::DrawPoint(int u, int v, int psize, 
                            unsigned int color) {

  for (int vv = v-psize/2; vv <= v+psize/2; vv++) {
    for (int uu = u-psize/2; uu <= u+psize/2; uu++) {
      SetGuarded(uu, vv, color);
    }
  }

}

void FrameBuffer::DrawCircle(float uf, float vf, 
  float r, unsigned int color) {

  float leftf = uf-r;
  float rightf = uf+r;
  float topf = vf-r;
  float bottomf = vf+r;

  if (rightf < 0.0f)
    return;
  if (rightf > (float) w)
    rightf = (float) w;
  if (leftf > (float)w)
    return;
  if (leftf < 0.0f)
    leftf = 0.0f;

  if (bottomf < 0.0f)
    return;
  if (topf > (float)h)
    return;

  int lefti = (int) (leftf+0.5f);
  int righti = (int) (rightf-0.5f);
  int topi = (int) (topf+0.5f);
  int bottomi = (int) (bottomf-0.5f);

  V3 center(uf, vf, 0.0f);
  for (int v = topi; v <= bottomi; v++) {
    for (int u = lefti; u <= righti; u++) {
      V3 pixv(.5f+(float)u, .5f+(float)v, 0.0f);
      if ((center-pixv).Length() > r)
        continue;
      Set(u, v, color);
    }
  }

}

void FrameBuffer::DrawSegment(V3 p0, V3 p1, 
  unsigned int color) {

    float du = fabsf(p0[0]-p1[0]);
    float dv = fabsf(p0[1]-p1[1]);
    int stepsN = (du > dv) ? (int) ceilf(du) : (int) ceilf(dv);
    for (int i = 0; i < stepsN; i++) {
      V3 p = p0 + (p1-p0)*((float)i/(float)(stepsN-1));
      int u = (int)p[0];
      int v = (int)p[1];
      SetGuarded(u, v, color);
    }

}
